unit Config;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, IniFiles;

type
  TConfigForm = class(TForm)
    BtnApply: TButton;
    BtnCancel: TButton;
    GroupBox1: TGroupBox;
    GroupBox2: TGroupBox;
    ChkReadOnly: TCheckBox;
    ChkHidden: TCheckBox;
    ChkSysFile: TCheckBox;
    GroupBox3: TGroupBox;
    MemIgnoreList: TMemo;
    EditSample: TEdit;
    UpdnSample: TUpDown;
    BtnMB: TButton;
    BtnKB: TButton;
    Label1: TLabel;
    EditLowerSize: TEdit;
    GroupBox4: TGroupBox;
    MemIgnoreExt: TMemo;
    GroupBox5: TGroupBox;
    MemRealExt: TMemo;
    GroupBox6: TGroupBox;
    Label2: TLabel;
    EditFastwardKey: TEdit;
    Label3: TLabel;
    EditFastwardRate: TEdit;
    UpdnFastwardRate: TUpDown;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure BtnApplyClick(Sender: TObject);
    procedure BtnCancelClick(Sender: TObject);
    procedure EditSampleExit(Sender: TObject);
    procedure BtnMBClick(Sender: TObject);
    procedure BtnKBClick(Sender: TObject);
    procedure EditLowerSizeExit(Sender: TObject);
    procedure EditFastwardKeyEnter(Sender: TObject);
    procedure EditFastwardRateExit(Sender: TObject);
  private

  public

  end;

type
    TConfig = class
    private
        FReadOnly: Boolean;
        FHidden: Boolean;
        FSysFile: Boolean;
        FIgnoreList: TStringList;
        FIgnoreExt: TStringList;
        FRealExt: TStringList;
        FLowerSize: Cardinal;
        FFastwardKey: Char;
        FFastwardRate: Integer;
    private
        FMRUItems: TStringList;
    public
        procedure AddMRUEntry(const APath: String);
        function GetMRUEntry(const Index: Integer): String;
        procedure RemoveMRUEntry(const APath: String);
        function MRUCount(): Integer;
    private
        procedure SetForm();
        procedure GetForm();
    public
        constructor Create();
        destructor Destroy(); override;
        function CheckFile(const AFileName: String): Boolean;
        function CheckReal(const AFileName: String): Boolean;
        procedure LoadConfig(const AIniPath: String);
        procedure SaveConfig(const AIniPath: String);
    public
        property FastwardKey: Char read FFastwardKey;
        property FastwardRate: Integer read FFastwardRate;
    end;

const
    MAX_MRU = 20;
    
var
    ConfigForm: TConfigForm;
    GConfig: TConfig = nil;

//  t@CTCY擾(FileSize)
function FileSizeByFileSize(const FileName: string): Cardinal;

implementation

{$R *.dfm}

const
    CHAR_FASTWARD: Char = 'F';

//=============================================================================
//  Utility
//=============================================================================

//-----------------------------------------------------------------------------
//  t@CTCY擾(FileSize)
function FileSizeByFileSize(const FileName: string): Cardinal;
var
    F: file;                            //  ^Ȃt@C
begin
    Result := 0;
    try
        FileMode := fmOpenRead or fmShareDenyNone;
        AssignFile(F, FileName);
        try
            Reset(F, 1);                //  oCgt@C
            Result := FileSize(F);
        finally
            CloseFile(F);
        end;
    except
    end;
end;

//=============================================================================
//  TConfig implementation
//=============================================================================

//-----------------------------------------------------------------------------
//  RXgN^
constructor TConfig.Create();
begin
    FReadOnly := False;
    FHidden := True;
    FSysFile := True;

    FIgnoreList := THashedStringList.Create();
    FIgnoreList.Sorted := True;
    FIgnoreList.Duplicates := dupIgnore;
    FIgnoreList.CaseSensitive := False;
    FIgnoreList.CommaText := 'Thumbs.db';

    FIgnoreExt := THashedStringList.Create();
    FIgnoreExt.Sorted := True;
    FIgnoreExt.Duplicates := dupIgnore;
    FIgnoreExt.CaseSensitive := False;
    FIgnoreExt.CommaText := '.db,.mov';

    FRealExt := THashedStringList.Create();
    FRealExt.Sorted := True;
    FRealExt.Duplicates := dupIgnore;
    FRealExt.CaseSensitive := False;
    FRealExt.CommaText := '.ram,.rm,.rmvb';

    FLowerSize := 500 * 1024;

    FFastwardKey := CHAR_FASTWARD;
    FFastwardRate := 10;

    FMRUItems := TStringList.Create();
end;

//-----------------------------------------------------------------------------
//  fXgN^
destructor TConfig.Destroy();
begin
    FMRUItems.Free();
    FIgnoreList.Free();
    FIgnoreExt.Free();
    FRealExt.Free();
end;

//-----------------------------------------------------------------------------
//  t@C󂯕t邩ǂ`FbN
function TConfig.CheckFile(const AFileName: String): Boolean;
var
    LAttr: Integer;
    LSize: Cardinal;
    LFileName: String;
    LFileExt: String;
begin
    Result := False;
    //  擾
    LAttr := SysUtils.FileGetAttr(AFileName);
    //  `FbN
    if (FReadOnly and ((LAttr and SysUtils.faReadOnly) <> 0)) then Exit;
    if (FHidden and ((LAttr and SysUtils.faHidden) <> 0)) then Exit;
    if (FSysFile and ((LAttr and SysUtils.faSysFile) <> 0)) then Exit;

    //  t@CTCY擾
    LSize := FileSizeByFileSize(AFileName);
    //  t@CTCY`FbN
    if (FLowerSize > LSize) then Exit;

    //  gq`FbN
    LFileExt := ExtractFileExt(AFileName);
    if (FIgnoreExt.IndexOf(LFileExt) <> -1) then Exit;

    //  t@C`FbN
    LFileName := ExtractFileName(AFileName);
    if (FIgnoreList.IndexOf(LFileName) <> -1) then Exit;

    //  I[O[
    Result := True;
end;

//-----------------------------------------------------------------------------
//  RealPlayer ɂ܂킷ǂ
function TConfig.CheckReal(const AFileName: String): Boolean;
var
    LFileExt: String;
begin
    LFileExt := ExtractFileExt(AFileName);
    Result := (FRealExt.IndexOf(LFileExt) <> -1);
end;

//-----------------------------------------------------------------------------
//  ݒe\
procedure TConfig.SetForm();
begin
    ConfigForm.ChkReadOnly.Checked := FReadOnly;    
    ConfigForm.ChkHidden.Checked := FHidden;
    ConfigForm.ChkSysFile.Checked := FSysFile;

    ConfigForm.MemIgnoreList.Text := Trim(FIgnoreList.Text);

    ConfigForm.MemIgnoreExt.Text := Trim(FIgnoreExt.Text);

    ConfigForm.MemRealExt.Text := Trim(FRealExt.Text);

    ConfigForm.EditLowerSize.Text := IntToStr(FLowerSize);
    ConfigForm.EditLowerSize.Tag := FLowerSize;

    ConfigForm.EditFastwardKey.Text := FFastwardKey;
    ConfigForm.UpdnFastwardRate.Position := FFastwardRate;
end;

//-----------------------------------------------------------------------------
//  ͓e
procedure TConfig.GetForm();
var
    i: Integer;
begin
    FReadOnly := ConfigForm.ChkReadOnly.Checked;
    FHidden := ConfigForm.ChkHidden.Checked;
    FSysFile := ConfigForm.ChkSysFile.Checked;

    FIgnoreList.Clear();
    for i := 0 to ConfigForm.MemIgnoreList.Lines.Count - 1 do
    begin
        FIgnoreList.Add(Trim(ConfigForm.MemIgnoreList.Lines[i]));
    end;

    FIgnoreExt.Clear();
    for i := 0 to ConfigForm.MemIgnoreExt.Lines.Count - 1 do
    begin
        FIgnoreExt.Add(Trim(ConfigForm.MemIgnoreExt.Lines[i]));
    end;

    FRealExt.Clear();
    for i := 0 to ConfigForm.MemRealExt.Lines.Count - 1 do
    begin
        FRealExt.Add(Trim(ConfigForm.MemRealExt.Lines[i]));
    end;

    FLowerSize := ConfigForm.EditLowerSize.Tag;

    if (ConfigForm.EditFastwardKey.Text <> '') then
    begin
        FFastwardKey := (ConfigForm.EditFastwardKey.Text)[1];
    end;

    FFastwardRate := ConfigForm.UpdnFastwardRate.Position;
end;

//-----------------------------------------------------------------------------
//  ŋߎgpXɒǉ
procedure TConfig.AddMRUEntry(const APath: String);
begin
    RemoveMRUEntry(APath);
    if DirectoryExists(APath) then
    begin
        FMRUItems.Insert(0, APath);
        while (FMRUItems.Count > MAX_MRU) do
        begin
            FMRUItems.Delete(FMRUItems.Count - 1);
        end;
    end;
end;

//-----------------------------------------------------------------------------
//  ŋߎgpX擾
function TConfig.GetMRUEntry(const Index: Integer): String;
begin
    Result := '';
    if (Index >= 0) and (Index < FMRUItems.Count) then
    begin
        Result := FMRUItems[Index];
    end;
end;

//-----------------------------------------------------------------------------
//  ŋߎgpX
procedure TConfig.RemoveMRUEntry(const APath: String);
var
    i: Integer;
begin
    for i := FMRUItems.Count - 1 downto 0 do
    begin
        if (AnsiCompareText(APath, FMRUItems[i]) = 0) then
        begin
            FMRUItems.Delete(i);
        end;
    end;
end;

//-----------------------------------------------------------------------------
//  ŋߎgpX̐
function TConfig.MRUCount(): Integer;
begin
    Result := FMRUItems.Count;
end;

//-----------------------------------------------------------------------------
//  ݒǍ
procedure TConfig.LoadConfig(const AIniPath: String);
var
    LIniFile: TIniFile;
    LInt: Integer;
    i: Integer;
    LText: String;
begin
    if not FileExists(AIniPath) then SaveConfig(AIniPath);

    LIniFile := TIniFile.Create(AIniPath);
    try
        //  鑮ǂݍ
        FReadOnly := LIniFile.ReadBool('IgnoreAttr', 'FReadOnly', FReadOnly);
        FHidden := LIniFile.ReadBool('IgnoreAttr', 'FHidden', FHidden);
        FSysFile := LIniFile.ReadBool('IgnoreAttr', 'FSysFile', FSysFile);

        //  t@Cǂݍ
        LInt := LIniFile.ReadInteger('IgnoreList', 'ListCount', 0);
        FIgnoreList.Clear();
        for i := 0 to LInt - 1 do
        begin
            FIgnoreList.Add(LIniFile.ReadString('IgnoreList', IntToStr(i), ''));
        end;

        //  gqǂݍ
        LInt := LIniFile.ReadInteger('IgnoreExt', 'ListCount', 0);
        FIgnoreExt.Clear();
        for i := 0 to LInt - 1 do
        begin
            FIgnoreExt.Add(LIniFile.ReadString('IgnoreExt', IntToStr(i), ''));
        end;

        //  RealPlayerp̊gqǂݍ
        LInt := LIniFile.ReadInteger('RealExt', 'ListCount', 0);
        FRealExt.Clear();
        for i := 0 to LInt - 1 do
        begin
            FRealExt.Add(LIniFile.ReadString('RealExt', IntToStr(i), ''));
        end;

        //  TCYǂݍ
        FLowerSize := LIniFile.ReadInteger('LowerSize', 'FLowerSize', FLowerSize);

        //  񂵗pL[ǂݍ
        LText := LIniFile.ReadString('Fastward', 'FFastwardKey', CHAR_FASTWARD);
        if (LText <> '') then
        begin
            FFastwardKey := LText[1];
        end;

        //  񂵑xǂݍ
        FFastwardRate := LIniFile.ReadInteger('Fastward', 'FFastwardRate', FFastwardRate);

        //  ŋߎgtH_
        for i := MAX_MRU downto 1 do
        begin
            LText := LIniFile.ReadString('MRUItems', Format('MRUItem%d', [i]), '');
            if DirectoryExists(LText) then
            begin
                AddMRUEntry(LText);
            end;
        end;
    finally
        LIniFile.Free();
    end;
end;

//-----------------------------------------------------------------------------
//  ݒۑ
procedure TConfig.SaveConfig(const AIniPath: String);
var
    LIniFile: TIniFile;
    i: Integer;
    LText: String;
begin
    LIniFile := TIniFile.Create(AIniPath);
    try
        //  鑮ۑ
        LIniFile.EraseSection('IgnoreAttr');
        LIniFile.WriteBool('IgnoreAttr', 'FReadOnly', FReadOnly);
        LIniFile.WriteBool('IgnoreAttr', 'FHidden', FHidden);
        LIniFile.WriteBool('IgnoreAttr', 'FSysFile', FSysFile);

        //  t@Cۑ
        LIniFile.EraseSection('IgnoreList');
        LIniFile.WriteInteger('IgnoreList', 'ListCount', FIgnoreList.Count);
        for i := 0 to FIgnoreList.Count - 1 do
        begin
            LIniFile.WriteString('IgnoreList', IntToStr(i), FIgnoreList[i]);
        end;

        //  gqۑ
        LIniFile.EraseSection('IgnoreExt');
        LIniFile.WriteInteger('IgnoreExt', 'ListCount', FIgnoreExt.Count);
        for i := 0 to FIgnoreExt.Count - 1 do
        begin
            LIniFile.WriteString('IgnoreExt', IntToStr(i), FIgnoreExt[i]);
        end;

        //  RealPlayerp̊gqۑ
        LIniFile.EraseSection('RealExt');
        LIniFile.WriteInteger('RealExt', 'ListCount', FRealExt.Count);
        for i := 0 to FRealExt.Count - 1 do
        begin
            LIniFile.WriteString('RealExt', IntToStr(i), FRealExt[i]);
        end;

        //  TCYۑ
        LIniFile.EraseSection('LowerSize');
        LIniFile.WriteInteger('LowerSize', 'FLowerSize', FLowerSize);

        //  񂵗pL[ۑ
        LIniFile.EraseSection('Fastward');
        LIniFile.WriteString('Fastward', 'FFastwardKey', FFastwardKey);

        //  񂵑xǂݍ
        LIniFile.WriteInteger('Fastward', 'FFastwardRate', FFastwardRate);

        //  ŋߎgtH_
        LIniFile.EraseSection('MRUItems');
        for i := 1 to MAX_MRU do
        begin
            LText := GetMRUEntry(i - 1);
            if DirectoryExists(LText) then
            begin
                LIniFile.WriteString('MRUItems', Format('MRUItem%d', [i]), LText);
            end;
        end;
    finally
        LIniFile.Free();
    end;
end;


//=============================================================================
//  TConfigForm implementation
//=============================================================================

//-----------------------------------------------------------------------------
//  tH[쐬
procedure TConfigForm.FormCreate(Sender: TObject);
begin
    //
end;

//-----------------------------------------------------------------------------
//  tH[j
procedure TConfigForm.FormDestroy(Sender: TObject);
begin
    //
end;

//-----------------------------------------------------------------------------
//  tH[\
procedure TConfigForm.FormShow(Sender: TObject);
begin
    GConfig.SetForm();
end;

//-----------------------------------------------------------------------------
//  Kp
procedure TConfigForm.BtnApplyClick(Sender: TObject);
begin
    GConfig.GetForm();
    Self.Close();
end;

//-----------------------------------------------------------------------------
//  LZ
procedure TConfigForm.BtnCancelClick(Sender: TObject);
begin
    Self.Close();
end;

//-----------------------------------------------------------------------------
//  TCYl̓`FbN
procedure TConfigForm.EditLowerSizeExit(Sender: TObject);
var
    LEdit: TEdit;
begin
    LEdit := TEdit(Sender);
    if (StrToIntDef(LEdit.Text, -1) < 0) then
    begin
        LEdit.Text := IntToStr(LEdit.Tag);
    end;
    LEdit.Tag := StrToIntDef(LEdit.Text, 0);
end;

//-----------------------------------------------------------------------------
//  Tv{bNX̓`FbN
procedure TConfigForm.EditSampleExit(Sender: TObject);
var
    LEdit: TEdit;
begin
    LEdit := TEdit(Sender);
    if (StrToIntDef(LEdit.Text, -1) < 0) then
    begin
        LEdit.Text := IntToStr(UpdnSample.Position);
    end;
end;

//-----------------------------------------------------------------------------
//  Tv(KB)
procedure TConfigForm.BtnKBClick(Sender: TObject);
begin
    EditLowerSize.Tag := UpdnSample.Position * 1024;
    EditLowerSize.Text := IntToStr(EditLowerSize.Tag);
end;

//-----------------------------------------------------------------------------
//  Tv(MB)
procedure TConfigForm.BtnMBClick(Sender: TObject);
begin
    EditLowerSize.Tag := UpdnSample.Position * 1024 * 1024;
    EditLowerSize.Text := IntToStr(EditLowerSize.Tag);
end;

//-----------------------------------------------------------------------------
//  tH[JXSI
procedure TConfigForm.EditFastwardKeyEnter(Sender: TObject);
begin
    TEdit(Sender).SelectAll();
end;

//-----------------------------------------------------------------------------
//  񂵑x̓`FbN
procedure TConfigForm.EditFastwardRateExit(Sender: TObject);
var
    LEdit: TEdit;
    LVal: Integer;
begin
    LEdit := TEdit(Sender);
    LVal := StrToIntDef(LEdit.Text, -1);
    if (LVal < UpdnFastwardRate.Min) or (UpdnFastwardRate.Max < LVal) then
    begin
        LEdit.Text := IntToStr(UpdnFastwardRate.Position);
    end;
end;


end.
